@@ -178,6 +178,24 @@ int fd;
 
 }
 
+static
+void
+expSetpgrp()
+{
+#ifdef MIPS_BSD
+    /* required on BSD side of MIPS OS <jmsellen@watdragon.waterloo.edu> */
+#   include <sysv/sys.s>
+    syscall(SYS_setpgrp);
+#endif
+
+#ifdef SETPGRP_VOID
+    (void) setpgrp();
+#else
+    (void) setpgrp(0,0);
+#endif
+}
+
+
 /* returns fd of master side of pty */
 int
 exp_spawnv(file,argv)
@@ -224,7 +242,7 @@ when trapping, see below in child half of fork */
 	}
 	fcntl(exp_pty[0],F_SETFD,1);	/* close on exec */
 #ifdef PTYTRAP_DIES
-	exp_slave_control(exp_pty[0],1);*/
+	exp_slave_control(exp_pty[0],1);
 #endif
 
 	if (!fd_new(exp_pty[0])) {
@@ -328,16 +346,10 @@ when trapping, see below in child half of fork */
 #else
 #ifdef SYSV3
 #ifndef CRAY
-	setpgrp();
+	expSetpgrp();
 #endif /* CRAY */
 #else /* !SYSV3 */
-#ifdef MIPS_BSD
-	/* required on BSD side of MIPS OS <jmsellen@watdragon.waterloo.edu> */
-#	include <sysv/sys.s>
-	syscall(SYS_setpgrp);
-#endif
-	setpgrp(0,0);
-/*	setpgrp(0,getpid());*/	/* make a new pgrp leader */
+	expSetpgrp();
 
 #ifdef TIOCNOTTY
 	ttyfd = open("/dev/tty", O_RDWR);
@@ -1171,18 +1183,13 @@ exp_disconnect()
 #else
 #ifdef SYSV3
 	/* put process in our own pgrp, and lose controlling terminal */
-	setpgrp();
+	expSetpgrp();
 	signal(SIGHUP,SIG_IGN);
 	if (fork()) exit(0);	/* first child exits (as per Stevens, */
 	/* UNIX Network Programming, p. 79-80) */
 	/* second child process continues as daemon */
 #else /* !SYSV3 */
-#ifdef MIPS_BSD
-	/* required on BSD side of MIPS OS <jmsellen@watdragon.waterloo.edu> */
-#	include <sysv/sys.s>
-	syscall(SYS_setpgrp);
-#endif
-	setpgrp(0,getpid());	/* put process in our own pgrp */
+	expSetpgrp();
 /* Pyramid lacks this defn */
 #ifdef TIOCNOTTY
 	ttyfd = open("/dev/tty", O_RDWR);
