@@ -28,6 +28,8 @@
 
  * *** and again for Expect!!! - DEL
 
+ * *** More minor corrections stolen from tcl7.5p1/regexp.c - DEL
+
  */
 
 #include "tcl.h"
@@ -35,6 +37,7 @@
 #include "exp_prog.h"
 #include "tclRegexp.h"
 #include "exp_regexp.h"
+#include "string.h"
 
 #define NOTSTATIC	/* was at one time, but Expect needs access */
 
@@ -89,7 +92,7 @@
 #define	PLUS	11	/* node	Match this (simple) thing 1 or more times. */
 #define	OPEN	20	/* no	Mark this point in input as start of #n. */
 			/*	OPEN+1 is number 1, etc. */
-#define	CLOSE	30	/* no	Analogous to OPEN. */
+#define	CLOSE	(OPEN+NSUBEXP)	/* no	Analogous to OPEN. */
 
 /*
  * Opcode notes:
@@ -417,7 +420,7 @@ int *flagp;
 						if (clss > classend+1)
 							FAIL("invalid [] range");
 						for (; clss <= classend; clss++)
-							regc(clss);
+							regc((char)clss);
 						regparse++;
 					}
 				} else
@@ -488,7 +491,7 @@ int *flagp;
  */
 static char *			/* Location. */
 regnode(op)
-char op;
+int op;
 {
 	register char *ret;
 	register char *ptr;
@@ -500,7 +503,7 @@ char op;
 	}
 
 	ptr = ret;
-	*ptr++ = op;
+	*ptr++ = (char)op;
 	*ptr++ = '\0';		/* Null "next" pointer. */
 	*ptr++ = '\0';
 	regcode = ptr;
@@ -513,10 +516,10 @@ char op;
  */
 static void
 regc(b)
-char b;
+int b;
 {
 	if (regcode != &regdummy)
-		*regcode++ = b;
+		*regcode++ = (char)b;
 	else
 		regsize++;
 }
@@ -528,7 +531,7 @@ char b;
  */
 static void
 reginsert(op, opnd)
-char op;
+int op;
 char *opnd;
 {
 	register char *src;
@@ -547,9 +550,9 @@ char *opnd;
 		*--dst = *--src;
 
 	place = opnd;		/* Op node, where operand used to be. */
-	*place++ = op;
-	*place++ = '\0';
+	*place++ = (char)op;
 	*place++ = '\0';
+	*place = '\0';
 }
 
 /*
@@ -580,8 +583,8 @@ char *val;
 		offset = scan - val;
 	else
 		offset = val - scan;
-	*(scan+1) = (offset>>8)&0377;
-	*(scan+2) = offset&0377;
+	*(scan+1) = (char)(offset>>8)&0377;
+	*(scan+2) = (char)offset&0377;
 }
 
 /*
@@ -774,7 +777,9 @@ char *prog;
 {
 	register char *scan;	/* Current node. */
 	char *next;		/* Next node. */
+#ifndef strchr	/* May be #defined to something else */
 	extern char *strchr();
+#endif
 
 	scan = prog;
 #ifdef DEBUG
@@ -864,6 +869,7 @@ char *prog;
 				register char *save;
 				int r;	/* result of regmatch */
 
+	doOpen:
 				no = OP(scan) - OPEN;
 				save = reginput;
 
@@ -894,6 +900,7 @@ char *prog;
 				register char *save;
 				int r;	/* result of regmatch */
 
+	doClose:
 				no = OP(scan) - CLOSE;
 				save = reginput;
 
@@ -946,6 +953,8 @@ char *prog;
 				register int min;
 				int match_status;
 
+				if (*reginput == '\0') return EXP_CANMATCH;
+
 				/*
 				 * Lookahead to avoid useless match attempts
 				 * when we know what character comes next.
@@ -965,7 +974,8 @@ char *prog;
 						if (r == EXP_MATCH)
 							return(EXP_MATCH);
 						if (r == EXP_CANMATCH)
-							match_status = r;
+/*							match_status = r;*/
+							return(EXP_CANMATCH);
 					}
 					/* Couldn't or didn't -- back up. */
 					no--;
@@ -976,10 +986,21 @@ char *prog;
 			/* NOTREACHED */
 			break;
 		case END:
-			return(EXP_MATCH);	/* Success! */
+			/* Success! */
+			if (*reginput == '\0') {
+				return(EXP_CANMATCH);
+			} else {
+				return(EXP_MATCH);
+			}
+/*			return(EXP_CANMATCH);	/* Success! */
 			/* NOTREACHED */
 			break;
 		default:
+			if (OP(scan) > OPEN && OP(scan) < OPEN+NSUBEXP) {
+				goto doOpen;
+			} else if (OP(scan) > CLOSE && OP(scan) < CLOSE+NSUBEXP) {
+				goto doClose;
+			}
 			regerror("memory corruption");
 			return(EXP_TCLERROR);
 			/* NOTREACHED */
@@ -1007,7 +1028,9 @@ char *p;
 	register int count = 0;
 	register char *scan;
 	register char *opnd;
+#ifndef strchr	/* May be #defined to something else */
 /*DEL*/	extern char *strchr();
+#endif
 
 	scan = reginput;
 	opnd = OPERAND(p);
@@ -1188,7 +1211,16 @@ char *op;
 		p = "PLUS";
 		break;
 	default:
-		regerror("corrupted opcode");
+		if (OP(op) > OPEN && OP(op) < OPEN+NSUBEXP) {
+		    sprintf(buf+strlen(buf), "OPEN%d", OP(op)-OPEN);
+		    p = NULL;
+		    break;
+		} else if (OP(op) > CLOSE && OP(op) < CLOSE+NSUBEXP) {
+		    sprintf(buf+strlen(buf), "CLOSE%d", OP(op)-CLOSE);
+		    p = NULL;
+		} else {
+		    TclRegError("corrupted opcode");
+		}
 		break;
 	}
 	if (p != NULL)
